from llmterface_gemini.config import GeminiConfig, AllowedGeminiModels
from llmterface_gemini.chat import GeminiChat
from llmterface_gemini.models import (
    GeminiTextModelType,
    GeminiAudioModelType,
    GeminiEmbeddingModelType,
    GeminiImageModelType,
    GeminiVideoModelType,
)

__all__ = [
    "GeminiConfig",
    "AllowedGeminiModels",
    "GeminiChat",
    "GeminiTextModelType",
    "GeminiAudioModelType",
    "GeminiEmbeddingModelType",
    "GeminiImageModelType",
    "GeminiVideoModelType",
]



from llmterface.providers.provider_spec import ProviderSpec
from llmterface_gemini.config import GeminiConfig
from llmterface_gemini.chat import GeminiChat

PROVIDER = ProviderSpec(
    provider=GeminiConfig.PROVIDER,
    config_cls=GeminiConfig,
    chat_cls=GeminiChat,
)



import typing as t

from google.genai.types import GenerateContentResponse
from google.genai.client import Client as GenaiClient
from google.genai.chats import Chat as GenaiChat

from llmterface.models.question import Question
from llmterface.models.generic_response import GenericResponse
from llmterface.providers.provider_chat import ProviderChat
from llmterface_gemini.config import (
    GeminiConfig,
)


def convert_response_to_generic(
    response: GenerateContentResponse,
) -> GenericResponse[GenerateContentResponse]:
    return GenericResponse(
        original=response,
        text=response.text or "",
    )


class GeminiChat(ProviderChat):

    def __init__(
        self,
        id: str,
        config: t.Optional[GeminiConfig] = None,
        client: t.Optional[GenaiClient] = None,
        sdk_chat: t.Optional[GenaiChat] = None,
    ):
        self.id = id
        self.config = config
        self.client = client
        self.sdk_chat = sdk_chat

    def ask(
        self, question: Question, client_config: t.Optional[GeminiConfig] = None
    ) -> GenericResponse:
        client_config = client_config or self.config
        if client_config is None:
            raise ValueError("GeminiConfig must be provided to ask a question.")
        if not self.sdk_chat:
            if self.client is None:
                self.client = GenaiClient(api_key=client_config.api_key)
            self.sdk_chat = self.client.chats.create(model=client_config.model.value)
        res = self.sdk_chat.send_message(
            question.prompt, config=client_config.gen_content_config
        )
        return convert_response_to_generic(res)



import typing as t
from pydantic import Field, field_validator
from google.genai.types import GenerateContentConfig


import llmterface as llm
from llmterface_gemini.models import GeminiTextModelType

AllowedGeminiModels = GeminiTextModelType


class GeminiConfig(llm.ProviderConfig):
    GENERIC_MODEL_MAPPING: t.ClassVar[
        dict[llm.GenericModelType, AllowedGeminiModels]
    ] = {
        llm.GenericModelType.text_lite: GeminiTextModelType.CHAT_2_0_FLASH_LITE,
        llm.GenericModelType.text_standard: GeminiTextModelType.CHAT_2_0_FLASH,
        llm.GenericModelType.text_heavy: GeminiTextModelType.CHAT_2_5_PRO,
    }
    DEFAULT_MODEL: t.ClassVar[AllowedGeminiModels] = GeminiTextModelType.CHAT_2_0_FLASH
    PROVIDER: t.ClassVar[str] = "gemini"
    api_key: str = Field(
        ..., description="API key for authenticating with the Gemini service."
    )
    model: GeminiTextModelType = Field(
        default=DEFAULT_MODEL, description="Gemini model to use for requests."
    )
    gen_content_config: GenerateContentConfig | None = Field(
        None,
        description="pre-configured GenerateContentConfig to use for requests.",
    )

    @classmethod
    def from_generic_config(
        cls,
        config: llm.GenericConfig,
    ) -> "GeminiConfig":
        gen_content_config = GenerateContentConfig(
            temperature=config.temperature,
            max_output_tokens=config.max_output_tokens,
            system_instruction=config.system_instruction,
            response_mime_type="application/json",
            response_json_schema=config.response_model.model_json_schema(),
        )
        return cls(
            api_key=config.api_key,
            model=config.model,
            gen_content_config=gen_content_config,
        )

    @field_validator("model", mode="before")
    @classmethod
    def validate_model(
        cls, v: AllowedGeminiModels | llm.GenericModelType | str | None
    ) -> t.Optional[GeminiTextModelType]:
        if v is None:
            return None
        if isinstance(v, AllowedGeminiModels):
            return v
        if isinstance(v, llm.GenericModelType):
            if model := cls.GENERIC_MODEL_MAPPING.get(v):
                return model
            raise NotImplementedError(f"No mapping for generic model type: {v}")
        try:
            return AllowedGeminiModels(v)
        except ValueError:
            raise ValueError(f"Invalid Gemini model type: {v}")



from enum import Enum


class GeminiEmbeddingModelType(Enum):
    EMBEDDING_GECKO_001 = (
        "embedding-gecko-001"  # Obtain a distributed representation of a text.
    )
    EMBEDDING_001 = "embedding-001"  # Obtain a distributed representation of a text.
    TEXT_EMBEDDING_004 = (
        "text-embedding-004"  # Obtain a distributed representation of a text.
    )
    GEMINI_EMBEDDING_EXP_03_07 = (
        "gemini-embedding-exp-03-07"  # Obtain a distributed representation of a text.
    )
    GEMINI_EMBEDDING_EXP = (
        "gemini-embedding-exp"  # Obtain a distributed representation of a text.
    )
    GEMINI_EMBEDDING_001 = (
        "gemini-embedding-001"  # Obtain a distributed representation of a text.
    )


class GeminiAudioModelType(Enum):
    CHAT_2_5_FLASH_NATIVE_AUDIO_LATEST = "gemini-2.5-flash-native-audio-latest"  # Latest release of Gemini 2.5 Flash Native Audio
    CHAT_2_5_FLASH_NATIVE_AUDIO_PREVIEW_09_2025 = "gemini-2.5-flash-native-audio-preview-09-2025"  # Gemini 2.5 Flash Native Audio Preview 09-2025
    CHAT_2_5_FLASH_NATIVE_AUDIO_PREVIEW_12_2025 = "gemini-2.5-flash-native-audio-preview-12-2025"  # Gemini 2.5 Flash Native Audio Preview 12-2025


class GeminiVideoModelType(Enum):
    VEO_2_0_GENERATE_001 = "veo-2.0-generate-001"  # Vertex served Veo 2 model. Access to this model requires billing to be enabled on the associated Google Cloud Platform account. Please visit https://console.cloud.google.com/billing to enable it.
    VEO_3_0_GENERATE_001 = "veo-3.0-generate-001"  # Veo 3
    VEO_3_0_FAST_GENERATE_001 = "veo-3.0-fast-generate-001"  # Veo 3 fast
    VEO_3_1_GENERATE_PREVIEW = "veo-3.1-generate-preview"  # Veo 3.1
    VEO_3_1_FAST_GENERATE_PREVIEW = "veo-3.1-fast-generate-preview"  # Veo 3.1 fast


class GeminiImageModelType(Enum):
    IMAGEN_4_0_GENERATE_PREVIEW_06_06 = (
        "imagen-4.0-generate-preview-06-06"  # Vertex served Imagen 4.0 model
    )
    IMAGEN_4_0_ULTRA_GENERATE_PREVIEW_06_06 = "imagen-4.0-ultra-generate-preview-06-06"  # Vertex served Imagen 4.0 ultra model
    IMAGEN_4_0_GENERATE_001 = (
        "imagen-4.0-generate-001"  # Vertex served Imagen 4.0 model
    )
    IMAGEN_4_0_ULTRA_GENERATE_001 = (
        "imagen-4.0-ultra-generate-001"  # Vertex served Imagen 4.0 ultra model
    )
    IMAGEN_4_0_FAST_GENERATE_001 = (
        "imagen-4.0-fast-generate-001"  # Vertex served Imagen 4.0 Fast model
    )


class GeminiTextModelType(Enum):
    CHAT_2_5_FLASH = "gemini-2.5-flash"  # Stable version of Gemini 2.5 Flash, our mid-size multimodal model that supports up to 1 million tokens, released in June of 2025.
    CHAT_2_5_PRO = (
        "gemini-2.5-pro"  # Stable release (June 17th, 2025) of Gemini 2.5 Pro
    )
    CHAT_2_0_FLASH_EXP = "gemini-2.0-flash-exp"  # Gemini 2.0 Flash Experimental
    CHAT_2_0_FLASH = "gemini-2.0-flash"  # Gemini 2.0 Flash
    CHAT_2_0_FLASH_001 = "gemini-2.0-flash-001"  # Stable version of Gemini 2.0 Flash, our fast and versatile multimodal model for scaling across diverse tasks, released in January of 2025.
    CHAT_2_0_FLASH_EXP_IMAGE_GENERATION = "gemini-2.0-flash-exp-image-generation"  # Gemini 2.0 Flash (Image Generation) Experimental
    CHAT_2_0_FLASH_LITE_001 = (
        "gemini-2.0-flash-lite-001"  # Stable version of Gemini 2.0 Flash-Lite
    )
    CHAT_2_0_FLASH_LITE = "gemini-2.0-flash-lite"  # Gemini 2.0 Flash-Lite
    CHAT_2_0_FLASH_LITE_PREVIEW_02_05 = "gemini-2.0-flash-lite-preview-02-05"  # Preview release (February 5th, 2025) of Gemini 2.0 Flash-Lite
    CHAT_2_0_FLASH_LITE_PREVIEW = "gemini-2.0-flash-lite-preview"  # Preview release (February 5th, 2025) of Gemini 2.0 Flash-Lite
    CHAT_EXP_1206 = (
        "gemini-exp-1206"  # Experimental release (March 25th, 2025) of Gemini 2.5 Pro
    )
    CHAT_2_5_FLASH_PREVIEW_TTS = (
        "gemini-2.5-flash-preview-tts"  # Gemini 2.5 Flash Preview TTS
    )
    CHAT_2_5_PRO_PREVIEW_TTS = (
        "gemini-2.5-pro-preview-tts"  # Gemini 2.5 Pro Preview TTS
    )
    GEMMA_3_1B_IT = "gemma-3-1b-it"
    GEMMA_3_4B_IT = "gemma-3-4b-it"
    GEMMA_3_12B_IT = "gemma-3-12b-it"
    GEMMA_3_27B_IT = "gemma-3-27b-it"
    GEMMA_3N_E4B_IT = "gemma-3n-e4b-it"
    GEMMA_3N_E2B_IT = "gemma-3n-e2b-it"
    CHAT_FLASH_LATEST = "gemini-flash-latest"  # Latest release of Gemini Flash
    CHAT_FLASH_LITE_LATEST = (
        "gemini-flash-lite-latest"  # Latest release of Gemini Flash-Lite
    )
    CHAT_PRO_LATEST = "gemini-pro-latest"  # Latest release of Gemini Pro
    CHAT_2_5_FLASH_LITE = "gemini-2.5-flash-lite"  # Stable version of Gemini 2.5 Flash-Lite, released in July of 2025
    CHAT_2_5_FLASH_IMAGE_PREVIEW = (
        "gemini-2.5-flash-image-preview"  # Gemini 2.5 Flash Preview Image
    )
    CHAT_2_5_FLASH_IMAGE = "gemini-2.5-flash-image"  # Gemini 2.5 Flash Preview Image
    CHAT_2_5_FLASH_PREVIEW_09_2025 = (
        "gemini-2.5-flash-preview-09-2025"  # Gemini 2.5 Flash Preview Sep 2025
    )
    CHAT_2_5_FLASH_LITE_PREVIEW_09_2025 = "gemini-2.5-flash-lite-preview-09-2025"  # Preview release (Septempber 25th, 2025) of Gemini 2.5 Flash-Lite
    CHAT_3_PRO_PREVIEW = "gemini-3-pro-preview"  # Gemini 3 Pro Preview
    CHAT_3_PRO_IMAGE_PREVIEW = (
        "gemini-3-pro-image-preview"  # Gemini 3 Pro Image Preview
    )
    CHAT_NANO_BANANA_PRO_PREVIEW = (
        "nano-banana-pro-preview"  # Gemini 3 Pro Image Preview
    )
    CHAT_ROBOTICS_ER_1_5_PREVIEW = (
        "gemini-robotics-er-1.5-preview"  # Gemini Robotics-ER 1.5 Preview
    )
    CHAT_2_5_COMPUTER_USE_PREVIEW_10_2025 = "gemini-2.5-computer-use-preview-10-2025"  # Gemini 2.5 Computer Use Preview 10-2025
    CHAT_DEEP_RESEARCH_PRO_PREVIEW_12_2025 = "deep-research-pro-preview-12-2025"  # Preview release (December 12th, 2025) of Deep Research Pro
    AQA = "aqa"  # Model trained to return answers to questions that are grounded in provided sources, along with estimating answerable probability.
